﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace _9627
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            LayerOverlay layerOverlay = new LayerOverlay();
            Feature grid1 = new Feature(new RectangleShape(-200, 0, 0, -200));
            Feature grid2 = new Feature(new RectangleShape(0, 200, 200, 0));
            Feature grid3 = new Feature(new RectangleShape(-200, 200, 0, 0));
            Feature grid4 = new Feature(new RectangleShape(0, 0, 200, -200));

            Collection<Feature> grids = new Collection<Feature>();
            grids.Add(grid1);
            grids.Add(grid2);
            grids.Add(grid3);
            grids.Add(grid4);

            InMemoryFeatureLayer baseLayer = new InMemoryFeatureLayer();
            baseLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City7;
            //baseLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City7("label");
            baseLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            baseLayer.Open();
            layerOverlay.Layers.Add(baseLayer);

            ClusterPointFeatureLayer layer = new ClusterPointFeatureLayer(grids, "label");
            layer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("label");
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layer.Open();
            layerOverlay.Layers.Add(layer);

            Random r = new Random();

            for (int i = 0; i < 100; i++)
            {
                double x = r.Next(-200, 200);
                double y = r.Next(-200, 200);

                string label = r.Next(0, 50).ToString();

                Feature tmpFeature = new Feature(x, y);
                tmpFeature.ColumnValues.Add("label", label);

                baseLayer.InternalFeatures.Add(tmpFeature);
                layer.InternalFeatures.Add(tmpFeature);
            }

            winformsMap1.Overlays.Add(layerOverlay);

            winformsMap1.CurrentExtent = baseLayer.GetBoundingBox();
            winformsMap1.Refresh();
        }
    }

    public class ClusterPointFeatureLayer : InMemoryFeatureLayer
    {
        private Collection<Feature> gridFeatures;
        private Dictionary<Feature, Collection<Feature>> dict;

        public string NumberColumn { get; set; }

        internal Collection<Feature> GridFeatures
        {
            get
            {
                return gridFeatures;
            }
            set
            {
                gridFeatures = value;
                InitGrideFeatureCollection();
            }
        }

        public ClusterPointFeatureLayer(Collection<Feature> gridFeatures, string numberColumn)
        {
            this.gridFeatures = gridFeatures;
            this.NumberColumn = numberColumn;
            InitGrideFeatureCollection();
        }

        private void InitGrideFeatureCollection()
        {
            if (dict == null)
            {
                dict = new Dictionary<Feature, Collection<Feature>>();
            }

            dict.Clear();

            foreach (Feature gridFeature in gridFeatures)
            {
                dict.Add(gridFeature, new Collection<Feature>());
            }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            ZoomLevel currentZoomLevel = ZoomLevelSet.GetZoomLevelForDrawing(canvas.CurrentWorldExtent, canvas.Width, canvas.MapUnit, canvas.Dpi);

            Collection<Feature> drawingFeatures = FeatureSource.GetFeaturesForDrawing(canvas.CurrentWorldExtent, canvas.Width, canvas.Height, ReturningColumnsType.AllColumns);

            foreach (Feature gridFeature in gridFeatures)
            {
                foreach (Feature drawingFeature in drawingFeatures)
                {
                    if (gridFeature.Contains(drawingFeature))
                    {
                        dict[gridFeature].Add(drawingFeature);
                        continue;
                    }
                }
            }

            currentZoomLevel.Draw(canvas, GetFeaturesFromGrids(), new Collection<SimpleCandidate>(), labelsInAllLayers);
        }

        private Collection<Feature> GetFeaturesFromGrids()
        {
            Collection<Feature> returnFeatures = new Collection<Feature>();

            foreach (var item in dict)
            {
                BaseShape shape = item.Key.GetBoundingBox().GetCenterPoint(); // Pick center point in sample

                int count = 0;
                double valueSum = 0;

                foreach (Feature f in item.Value)
                {
                    if (NumberColumn != null && NumberColumn != string.Empty && f.ColumnValues.ContainsKey(NumberColumn))
                    {
                        double columnValue = Double.Parse(f.ColumnValues[NumberColumn]);

                        count++;
                        valueSum += columnValue;
                    }
                }

                Feature tmpFeature = new Feature(shape);

                if (item.Value.Count > 0)
                {
                    double averageNum = valueSum / count;
                    tmpFeature.ColumnValues.Add(NumberColumn, averageNum.ToString());
                }

                returnFeatures.Add(tmpFeature);
            }

            return returnFeatures;
        }
    }
}
